/* wmciv.c
 *
 * Version 0.2
 *
 * By Alexandre BERAUD (a_beraud@lemel.fr)
 *
 */

#include <stdlib.h>
#include <stdio.h>
#include <X11/Xlib.h>
#include <X11/xpm.h>
#include <X11/Xutil.h>
#include <X11/extensions/shape.h>

#include "fond.xbm"
#include "fond.xpm"
#include "pics.xpm"
#include "options.xpm"
#include "alpha.xpm"

#define NB 2

Display *dpy;
Window root;
int screen;
GC gc;
unsigned long p_blanc;
unsigned long p_noir;
char sname[30]; /* Name of saved game file displayed */
char term[100]; /* Terminal emulator */
char server[100]; /* Server command line */
char client[100]; /* Client command line */
char type[100]; /* Saved games type */

Window win;
Window iconwin;
unsigned int borderwidth=1;

XWMHints wmhints;
XSizeHints sizehints;
XClassHint classHint;

Pixmap pixmask,fond;
Pixmap pics,options,alpha;
int ngame=0,sw=0,ns=1,nsmax,svsw;

/* ngame : type of game to launch in main panel
   sw : main panel or option panel ?
   ns : number of actual saved game in the temporary file
   nsmax : number of saved games in the temporary file
   svsw : (saveswitch) is the saved game switch on or off ? */


void DispGfx (Pixmap pxm, int xs, int ys, int w, int h, int xd, int yd)
{
 /* pxm = pixmap
    xs = source x
    ys = source y
    w = width
    h = height
    xd = destination x
    yd = destination y
 */

  XCopyArea(dpy, pxm, win, gc, xs, ys, w, h, xd, yd);
  XCopyArea(dpy, pxm, iconwin, gc, xs, ys, w, h, xd, yd);
}

void Load (int n)
{
  char c;
  int i,j;
  FILE * fichier;

 /* We get the nth name in the list */

  fichier=fopen("sgames.wmc","r");
  for (j=0;j<n;j++)
    fgets(sname,30,fichier);
  if (nsmax!=0)
    sname[strlen(sname)-1]='\0';

  fclose(fichier);
}

void LoadConfig ()
{
  char tmp[100],cmd[100];
  FILE * fichier;

 /* Loading configuration and removing carriage return */

  system("cp ~/.wmciv/config.wmc .");
  fichier=fopen("config.wmc","r");
  fgets(tmp,100,fichier);
  fgets(term,100,fichier);
  term[strlen(term)-1]='\0';
  fgets(tmp,100,fichier);
  fgets(server,100,fichier);
  server[strlen(server)-1]='\0';
  fgets(tmp,100,fichier);
  fgets(client,100,fichier);
  client[strlen(client)-1]='\0';
  fgets(tmp,100,fichier);
  fgets(type,100,fichier);
  type[strlen(type)-1]='\0';

  fclose(fichier);
  system("rm config.wmc");

}

void Write (char *str)
{
  int i,l;
  char c;

 /* We paint the text zone in black first */

  for (i=5;i<10;i++)
    DispGfx(options,0,0,56,1,4,i);

  l=strlen(str);
  for (i=0;i<l;i++)
  {
    c=str[i];
    if ((c>='A') && (c<='Z'))
      c='a'+c-'A';
    if ((c>='a') && (c<='z'))
      DispGfx(alpha,4*(c-'a'),0,4,5,5+(5*i),5);
    if ((c>='0') && (c<='9'))
      DispGfx(alpha,4*(c-'0')+108,0,4,5,5+(5*i),5);
    if (c=='.')
      DispGfx(alpha,104,0,4,5,5+(5*i),5);

 /* Do NOT display more than 11 characters */

    if (i==10)
      i=l;
  }

}

void DoExpose()
{

 /* Display main panel and current game picture */

  if (sw==0)
  {
    DispGfx(fond,0,0,56,56,4,4);
    DispGfx(pics,0,ngame*28,56,28,4,4);
  }

 /* Display options panel, pressed switch (if svsw==1) and current saved
    game */

  if (sw==1)
  {
    DispGfx(options,0,0,56,56,4,4);
    if (svsw==1)
      DispGfx(options,0,56,8,8,4,11);
    Load(ns);
    Write(sname);
  }
}

void DoButtonPress(XEvent ev)
{

 /* Test button press for main panel */

 if (sw==0)
 {
  if ( ev.xbutton.y >= 32 &&    /* Left arrow */
       ev.xbutton.y <  44 &&
       ev.xbutton.x >= 4  &&
       ev.xbutton.x <  32 &&
       ev.xbutton.button == Button1)
    DispGfx(fond,0,56,28,12,4,32);
  if ( ev.xbutton.y >= 32 &&   /* Right arrow */
       ev.xbutton.y <  44 &&
       ev.xbutton.x >= 32 &&
       ev.xbutton.x <  60 &&
       ev.xbutton.button == Button1)
    DispGfx(fond,28,56,28,12,32,32);
  if ( ev.xbutton.y >= 44 &&   /* SER button */
       ev.xbutton.y <  60 &&
       ev.xbutton.x >= 4  &&
       ev.xbutton.x <  32 &&
       ev.xbutton.button == Button1)
    DispGfx(fond,0,68,28,16,4,44);
  if ( ev.xbutton.y >= 44 &&   /* CLI button */
       ev.xbutton.y <  60 &&
       ev.xbutton.x >= 32  &&
       ev.xbutton.x <  60 &&
       ev.xbutton.button == Button1)
    DispGfx(fond,28,68,28,16,32,44);
 }

 /* Test button press for options panel */

 if (sw==1)
 {

  if ( ev.xbutton.y >= 11 &&   /* Left arrow */
       ev.xbutton.y <  19 &&
       ev.xbutton.x >= 12  &&
       ev.xbutton.x <  36 &&
       ev.xbutton.button == Button1)
    DispGfx(options,8,56,24,8,12,11);

  if ( ev.xbutton.y >= 11 &&   /* Right arrow */
       ev.xbutton.y <  19 &&
       ev.xbutton.x >= 36  &&
       ev.xbutton.x <  60 &&
       ev.xbutton.button == Button1)
    DispGfx(options,32,56,24,8,36,11);

 }

}

void DoButtonRelease(XEvent ev)
{
  char c,cmd[100],cmd2[100];
  FILE * fichier;

 /* Test third button to switch panel */

 if ( ev.xbutton.y >= 4 &&
      ev.xbutton.y <  60 &&
      ev.xbutton.x >= 4  &&
      ev.xbutton.x <  60 &&
      ev.xbutton.button == Button3)
 {
   if (sw==1)
   {
     system("rm sgames.wmc");
   }
   sw=1-sw;
   if (sw==1)
   {

 /* Can't fopen a file in another directory. That's why the temporary file
    is created in the current directory. Should be fixed *asap*   */

     strcpy(cmd,"find ");  /* Hope you've got the 'find' program (you should) */
     strcat(cmd,type);
     strcat(cmd," -fprint sgames.wmc");
     system(cmd);
     fichier=fopen("sgames.wmc","r");
     if (fichier==NULL)
       printf("Error while opening temporary file.\n");

 /* Now we check how many file names are stocked in the temporary file
    We do this everytime cause the directory may change */

     nsmax=0;
     do
     {
       c=fgetc(fichier);
       if (c=='\n')
         nsmax++;
     } while (c!=EOF);
     fclose(fichier);
   }

 /* This is no lame code, just for the case I add another panel */

 }

 /* Buttons tests for options panel */

 if (sw==1)
 {

 /* Test pression of the saved game switch */

  if ( ev.xbutton.y >= 11 &&
       ev.xbutton.y <  19 &&
       ev.xbutton.x >= 4  &&
       ev.xbutton.x <  12 &&
       ev.xbutton.button == Button1)
  {
    if (nsmax!=0)
      svsw=1-svsw;
  }

  if ( ev.xbutton.y >= 11 &&   /* Left arrow */
       ev.xbutton.y <  19 &&
       ev.xbutton.x >= 12  &&
       ev.xbutton.x <  36 &&
       ev.xbutton.button == Button1)
  {
    if (ns!=1)
      ns--;
  }

  if ( ev.xbutton.y >= 11 &&   /* Right arrow */
       ev.xbutton.y <  19 &&
       ev.xbutton.x >= 36  &&
       ev.xbutton.x <  60 &&
       ev.xbutton.button == Button1)
  {
    if (ns<nsmax)
      ns++;
  }

 }

 /* Buttons tests for main panel */

 if (sw==0)
 {
  if ( ev.xbutton.y >= 32 &&   /* Left arrow */
       ev.xbutton.y <  44 &&
       ev.xbutton.x >= 4  &&
       ev.xbutton.x <  32 &&
       ev.xbutton.button == Button1)
  {
    if (ngame!=0)
      ngame--;
    else
      ngame=NB;
  }
  if ( ev.xbutton.y >= 32 &&   /* Right arrow */
       ev.xbutton.y <  44 &&
       ev.xbutton.x >= 32 &&
       ev.xbutton.x <  60 &&
       ev.xbutton.button == Button1)
  {
    if (ngame!=NB)
      ngame++;
    else
      ngame=0;
  }
  if ( ev.xbutton.y >= 44 &&   /* SER button */
       ev.xbutton.y <  60 &&
       ev.xbutton.x >= 4  &&
       ev.xbutton.x <  32 &&
       ev.xbutton.button == Button1)
  {
    strcpy(cmd2,term);
    strcat(cmd2," ");
    strcat(cmd2,server);
    if (svsw==1)
    {
      strcat(cmd2," -f ");
      strcat(cmd2,sname);
    }
    if (ngame==0)
    {
      strcat(cmd2," -r ~/.wmciv/simple.wmc");
    }
    if (ngame==1)
    {
      strcat(cmd2," -r ~/.wmciv/favorite.wmc");
    }
    if (ngame==2)
    {
      strcat(cmd2," -r ~/.wmciv/ancients.wmc");
    }
    strcat(cmd2," &");
    system(cmd2);
  }
  if ( ev.xbutton.y >= 44 &&   /* CLI button */
       ev.xbutton.y <  60 &&
       ev.xbutton.x >= 32 &&
       ev.xbutton.x <  60 &&
       ev.xbutton.button == Button1)
  {
    strcpy(cmd,client);
    if (ngame==2)
    {
      strcat(cmd," --tiles ancients");
    }
    strcat(cmd," &");
    system(cmd);
  }

 }

 /* After all this mess, redraw window ! */

 DoExpose();

}

 /* Hereafter, general dockapp routines (should not be changed) */

void DoEvents()
{
  XEvent ev;

  do
  {
    XNextEvent(dpy,&ev);
    switch(ev.type)
    {
      case Expose:
        DoExpose();
        break;
      case ButtonPress:
        DoButtonPress(ev);
        break;
      case ButtonRelease:
        DoButtonRelease(ev);
        break;
    }
  }
  while (1);

}

void main(int argc, char *argv[])
{
  char *wname=argv[0];

  dpy=XOpenDisplay(NULL);
  screen=DefaultScreen(dpy);
  root=RootWindow(dpy,screen);
  p_blanc=WhitePixel(dpy,screen);
  p_noir=BlackPixel(dpy,screen);
  gc=XDefaultGC(dpy,screen);

  sizehints.x=0;
  sizehints.y=0;
  sizehints.width=64;
  sizehints.height=64;

  win=XCreateSimpleWindow(dpy,root,sizehints.x,sizehints.y,
      sizehints.width, sizehints.height, borderwidth, p_noir, p_blanc);
  iconwin=XCreateSimpleWindow(dpy,root,sizehints.x,sizehints.y,
      sizehints.width, sizehints.height, borderwidth, p_noir, p_blanc);
  sizehints.flags=USSize|USPosition;
  XSetWMNormalHints(dpy,win,&sizehints);

  wmhints.initial_state=WithdrawnState;
  wmhints.icon_window=iconwin;
  wmhints.icon_x=sizehints.x;
  wmhints.icon_y=sizehints.y;
  wmhints.window_group=win;
  wmhints.flags=StateHint|IconWindowHint|IconPositionHint|WindowGroupHint;
  XSetWMHints(dpy,win,&wmhints);

  classHint.res_name=wname;
  classHint.res_class=wname;
  XSetClassHint(dpy,win,&classHint);
  XSetCommand(dpy,win,argv,argc);
  pixmask=XCreateBitmapFromData(dpy,win,fond_bits,fond_width,fond_height);
  XShapeCombineMask(dpy,win,ShapeBounding,0,0,pixmask,ShapeSet);
  XShapeCombineMask(dpy,iconwin,ShapeBounding,0,0,pixmask,ShapeSet);

 /* Loading (actually, not a real load) the pixmaps */

  XpmCreatePixmapFromData(dpy,root,fond_xpm,&fond,NULL,NULL);
  XpmCreatePixmapFromData(dpy,root,pics_xpm,&pics,NULL,NULL);
  XpmCreatePixmapFromData(dpy,root,options_xpm,&options,NULL,NULL);
  XpmCreatePixmapFromData(dpy,root,alpha_xpm,&alpha,NULL,NULL);

  XSelectInput (dpy,win,ButtonPressMask|ButtonReleaseMask|                                     FocusChangeMask|KeyPressMask|ExposureMask ) ;
  XSelectInput (dpy,iconwin,ButtonPressMask|ButtonReleaseMask|                                     FocusChangeMask|KeyPressMask|ExposureMask ) ;

  XMapWindow(dpy,win);

  LoadConfig();
  strcpy(sname,".none.");

  DoEvents();


}




